<?php

namespace Drupal\external_entities\GuzzleHttp;

use Drupal\Core\Logger\LoggerChannelTrait;
use GuzzleHttp\Client;
use Psr\Http\Message\ResponseInterface;

/**
 * Debug version of Guzzle Client that logs query info in Drupal logs.
 */
class DebugClient extends Client {

  use LoggerChannelTrait;

  /**
   * Default logger channel name.
   *
   * @var string
   */
  protected $defaultLoggerChannel = 'xntt_guzzle';

  /**
   * {@inheritdoc}
   */
  public function request($method, $uri = '', array $options = []): ResponseInterface {
    $this->getLogger($this->defaultLoggerChannel)->debug(
      "METHOD: '$method'\nURI: '$uri'\nOPTIONS:\n{options}",
      ['options' => print_r($options, TRUE)]
    );
    $response = parent::request($method, $uri, $options);
    $status = $response->getStatusCode();
    $this->getLogger($this->defaultLoggerChannel)->debug(
      "STATUS CODE: "
      . $status
      . "\nHEADERS: {headers}",
      ['headers' => print_r($response->getHeaders(), TRUE)]
    );
    if (200 == $status) {
      $this->getLogger($this->defaultLoggerChannel)->debug(
        "BODY:\n{body}",
        ['body' => '' . $response->getBody()]
      );
    }

    return $response;
  }

  /**
   * Sets the logger channel to use for logs.
   *
   * @param string $channel
   *   Channel name. Should be set to plugin channel name.
   */
  public function setDefaultLoggerChannel(string $channel) {
    $this->defaultLoggerChannel = $channel;
  }

}
